@@ -1360,11 +1360,12 @@ class KeyringPackages(_Admin, _Running, _Confirm, _CatAdministrator, Call):
         self.result = []
         events_queue = events.Attach(daemon.get().events)
 
+        package_version = package.DebianVersion.stamp()  # Use same value across distributions (else keyring package may not be dist-upgradeable)
         for diststr in self.args["distributions"].value():
             repository, distribution, suite = util.models().parse_diststr(diststr)
             if not suite.build_keyring_package:
                 raise util.HTTPBadRequest(f"Keyring package to non-keyring suite requested (see 'build_keyring_package' flag): '{diststr}'")
-            self.result.append(package.upload_template_package(package.KeyringPackage(), diststr))
+            self.result.append(package.upload_template_package(package.KeyringPackage(package_version=package_version), diststr))
 
         def unfinished():
             return [upload for upload in self.result if "error" not in upload and "event" not in upload]
