@@ -1,12 +1,13 @@
-import os
+import ftplib
 import glob
 import logging
-import ftplib
+import os
 import re
+import shutil
 
 import debian.deb822
 
-from mini_buildd import config, events, util, files, sbuild, dist, net
+from mini_buildd import config, daemon, dist, events, files, net, sbuild, util
 
 LOG = logging.getLogger(__name__)
 
@@ -25,10 +26,13 @@ class Base(debian.deb822.Changes):
 
     def __str__(self):
         """Superclass would show all values multiline, so we need to replace with s.th. less invasive"""
-        return f"{self.get('Source')}_{self.get('Version')} ({self.get('Architecture')})"
+        return self.dfn.changes(self.get("Architecture")) if self.file_path is None else self.file_name()
 
     def file_name(self):
-        return os.path.basename(self.file_path) if self.file_path is not None else None
+        return None if self.file_path is None else os.path.basename(self.file_path)
+
+    def dir_path(self):
+        return None if self.file_path is None else os.path.dirname(self.file_path)
 
     def dsc_file_path(self):
         return os.path.join(os.path.dirname(self.file_path), self.dfn.dsc())
@@ -38,11 +42,13 @@ class Base(debian.deb822.Changes):
 
     def add_file(self, file_name):
         self.setdefault("Files", [])
-        self["Files"].append({"md5sum": util.Hash(file_name).md5(),
-                              "size": os.path.getsize(file_name),
-                              "section": "mini-buildd",
-                              "priority": "extra",
-                              "name": os.path.basename(file_name)})
+        self["Files"].append({
+            "md5sum": util.Hash(file_name).md5(),
+            "size": os.path.getsize(file_name),
+            "section": "mini-buildd",
+            "priority": "extra",
+            "name": os.path.basename(file_name),
+        })
 
     def upload_file_path(self):
         return os.path.splitext(self.file_path)[0] + ".upload"
@@ -56,7 +62,7 @@ class Base(debian.deb822.Changes):
         upload = self.upload_file_path()
         if not force and os.path.exists(upload):
             with util.fopen(upload) as uf:
-                LOG.warning(f"FTP: '{upload}' exists (already uploaded to '{uf.read()}')")
+                LOG.warning("FTP: '%s' exists (already uploaded to '%s')", upload, uf.read())
         else:
             with ftp_cls() as ftp:
                 ftp.connect(endpoint.options.get("host"), int(endpoint.options.get("port")))
@@ -67,12 +73,12 @@ class Base(debian.deb822.Changes):
                 ftp.cwd("/incoming")
                 for fd in self.get_files() + [{"name": self.file_name()}]:
                     f = fd["name"]
-                    LOG.debug(f"FTP: Uploading file: '{f}'")
+                    LOG.debug("FTP: Uploading file: '%s'", f)
                     with open(os.path.join(os.path.dirname(self.file_path), f), "rb") as fi:
                         ftp.storbinary(f"STOR {f}", fi)
                 with util.fopen(upload, "w") as fi:
                     fi.write(endpoint.geturl())
-                LOG.info(f"FTP: '{self.file_name()}' uploaded to '{endpoint.geturl()}'.")
+                LOG.info("FTP: '%s' uploaded to '%s'.", self.file_name(), endpoint.geturl())
 
         return pre_upload_timestamp  # for events --after=
 
@@ -88,24 +94,31 @@ class Base(debian.deb822.Changes):
 
     def save_as(self, file_path, gnupg=None):
         """Write to file (optionally signed)"""
-        LOG.debug(f"Saving changes: {file_path}")
+        LOG.debug("Saving changes: %s", file_path)
         os.makedirs(os.path.dirname(file_path), exist_ok=True)  # Be sure base dir is always available
         with open(file_path, "w+", encoding=config.CHAR_ENCODING) as f:
             f.write(self.dump())
         if gnupg is not None:
-            LOG.debug(f"Signing changes: {file_path}")
+            LOG.debug("Signing changes: %s", file_path)
             gnupg.sign(file_path)
         self.file_path = file_path
 
-    def _move_to(self, path):
-        """Move changes file along with all accompanying files (needs ``file_path`` to be non-None)"""
-        LOG.debug(f"Moving changes: '{self.file_path}'->'{path.full}'")
+    def _move_to(self, dir_path, mv):
+        """Move (or copy) changes file along with all accompanying files (needs ``file_path`` to be non-None)"""
+        LOG.debug("%s changes: '%s'->'%s'", mv.__name__, self.file_path, dir_path)
+        os.makedirs(dir_path, exist_ok=True)
         for fd in [{"name": self.file_name()}] + self.get_files():
             f = fd["name"]
-            f_abs = os.path.join(os.path.dirname(self.file_path), f)
-            LOG.debug(f"Moving '{f}' to '{path.full}'")
-            os.rename(f_abs, os.path.join(path.full, f))
-        self.file_path = path.join(path.full, self.file_name())
+            f_abs = os.path.join(self.dir_path(), f)
+            LOG.debug("%s '{f}' to '%s'", mv.__name__, dir_path)
+            mv(f_abs, os.path.join(dir_path, f))
+        self.file_path = os.path.join(dir_path, self.file_name())
+
+    def move_to(self, path):
+        return self._move_to(path, mv=os.rename)
+
+    def copy_to(self, path):
+        return self._move_to(path, mv=shutil.copy)
 
 
 class Changes(Base):
@@ -132,8 +145,8 @@ class Changes(Base):
         #: Key to identify a 'Build' (Buildrequest, Buildresult).
         self.bkey = os.path.join(self.key, self["Architecture"])
 
-        self.events_path = config.ROUTES["events"].path.new_sub([self.bkey])
-        self.builds_path = config.ROUTES["builds"].path.new_sub([self.bkey])
+    def get_events_path(self, *args):
+        return os.path.join(config.ROUTES["events"].path.full, self.bkey, *args)
 
     @property
     def timecode(self):
@@ -159,29 +172,14 @@ class Changes(Base):
             **{field.snake_name: value for field, value in self.fields().items() if field.is_cfield or field.snake_name in ["architecture", "maintainer", "changed_by"]},
         }
 
-    def save_to(self, path, tar=None):
-        file_path = path.join(self.dfn.changes(self["Architecture"], self.FILE_APX))
+    def save_to(self, dir_path, tar=None):
+        os.makedirs(dir_path, exist_ok=True)
+        file_path = os.path.join(dir_path, self.dfn.changes(self["Architecture"], self.FILE_APX))
         if tar is not None:
             tar_path = file_path + ".tar"
             tar.save_as(tar_path)
             self.add_file(tar_path)
-        self.save_as(file_path, util.daemon().gnupg)
-
-    def create_events(self):
-        self.events_path.create()
-        return self
-
-    def move_to_events(self):
-        self.create_events()
-        return self._move_to(self.events_path)
-
-    def create_builds(self):
-        self.builds_path.create()
-        return self
-
-    def move_to_builds(self):
-        self.create_builds()
-        return self._move_to(self.builds_path)
+        self.save_as(file_path, daemon.get().gnupg)
 
 
 class Upload(Changes):
@@ -243,19 +241,20 @@ class Upload(Changes):
 
         @classmethod
         def usage(cls):
-            return ("(lintian|piuparts|autopkgtest)-mode[<arch>]=<mode>\n"
-                    f"  {dist.SbuildCheck.usage()}\n"
-                    "internal-apt-priority[<arch>]=<prio>\n"
-                    "  Apt prio for our repo packages when building (use 500 to always prefer).\n"
-                    "auto-ports=<dist>,...\n"
-                    "  List of distributions (comma-separated) to automatically run ports for after successful install.\n"
-                    "deb-build-options[<arch>]=<option> <option>...\n"
-                    "  Contents of DEB_BUILD_OPTIONS environment for building (see https://www.debian.org/doc/debian-policy/ch-source.html#debian-rules-and-deb-build-options).\n"
-                    "deb-build-profiles[<arch>]=<profile> <profile>...\n"
-                    "  Contents of DEB_BUILD_PROFILES environment for building (see https://wiki.debian.org/BuildProfileSpec).\n"
-                    "add-depends[<arch>]=<dep>,<dep>...\n"
-                    "  List of additional dependencies (comma-separated, like in ``debian/control``) (see ``sbuild --add-depends``).\n"
-                    )
+            return (
+                "(lintian|piuparts|autopkgtest)-mode[<arch>]=<mode>\n"
+                f"  {dist.SbuildCheck.usage()}\n"
+                "internal-apt-priority[<arch>]=<prio>\n"
+                "  Apt prio for our repo packages when building (use 500 to always prefer).\n"
+                "auto-ports=<dist>,...\n"
+                "  List of distributions (comma-separated) to automatically run ports for after successful install.\n"
+                "deb-build-options[<arch>]=<option> <option>...\n"
+                "  Contents of DEB_BUILD_OPTIONS environment for building (see https://www.debian.org/doc/debian-policy/ch-source.html#debian-rules-and-deb-build-options).\n"
+                "deb-build-profiles[<arch>]=<profile> <profile>...\n"
+                "  Contents of DEB_BUILD_PROFILES environment for building (see https://wiki.debian.org/BuildProfileSpec).\n"
+                "add-depends[<arch>]=<dep>,<dep>...\n"
+                "  List of additional dependencies (comma-separated, like in ``debian/control``) (see ``sbuild --add-depends``).\n"
+            )
 
         @classmethod
         def _get_top_changes(cls, upload_changes):
@@ -279,7 +278,7 @@ class Upload(Changes):
         def _parse_ancient(self):
             """Compat parse support for ancient 'magic' options"""
             def warning(ancient, option):
-                LOG.warning(f"Ancient user option \"{ancient}\" found. Please use option \"{option}\" instead (see user manual).")
+                LOG.warning("Ancient user option \"%s\" found. Please use option \"%s\" instead (see user manual).", ancient, option)
 
             ancient_auto_backports = re.search(r"\*\s*MINI_BUILDD:\s*AUTO_BACKPORTS:\s*([^*.\[\]]+)", self._top_changes)
             if ancient_auto_backports:
@@ -334,11 +333,11 @@ class Upload(Changes):
                 elif base_key == "run-lintian":
                     key = f"lintian-mode{alt}"
                     converted_value.value = "errfail" if oldvalue else "disabled"
-                LOG.warning(f"User Option: Converted DEPRECATED {oldkey}={oldvalue} to {key}={converted_value.value}")
+                LOG.warning("User Option: Converted DEPRECATED %s=%s to %s=%s", oldkey, oldvalue, key, converted_value.value)
 
             self._options[key] = converted_value.value
 
-            LOG.debug(f"Upload option set: {key}=\"{value}\"")
+            LOG.debug("Upload option set: %s=\"%s\"", key, value)
 
         def get(self, key, alt=None, default=None):
             """Get first existing option value in this order: key[a], key, default"""
@@ -368,8 +367,11 @@ class Upload(Changes):
 
     def request_builds(self, repository, distribution, suite_option):
         """Build buildrequest files for all architectures"""
-        with util.fopen(self.dsc_file_path()) as dsc_file:
-            dsc = debian.deb822.Dsc(dsc_file)
+        try:
+            with util.fopen(self.dsc_file_path()) as dsc_file:
+                dsc = debian.deb822.Dsc(dsc_file)
+        except FileNotFoundError as e:
+            raise util.HTTPBadRequest(f"Missing DSC file '{self.dfn.dsc()}' (Maybe ``dpkg-buildpackage`` was called with '-b/-B'?)") from e
 
         # Get all DSC/source package files:
         # - Check md5 against possible pool files.
@@ -387,13 +389,14 @@ class Upload(Changes):
                 files_from_dsc.append(os.path.join(os.path.dirname(self.dsc_file_path()), f["name"]))
             elif from_pool is not None:
                 files_from_dsc.append(from_pool)
-                LOG.info(f"{self}: File added from pool: {from_pool}")
+                LOG.info("%s: File added from pool: %s", self, from_pool)
             else:
                 raise util.HTTPBadRequest(f"Missing file '{f['name']}' neither in upload, nor in pool (use '-sa' for uploads with new upstream)")
 
         buildrequest_dict = {}
         for ao in distribution.architectureoption_set.all():
-            path = self.events_path.new_sub([ao.architecture.name], create=True)   # Usually: <source>/<version>/<timecode>/source/<arch>/
+            path = os.path.join(self.dir_path(), ao.architecture.name)   # Usually: <source>/<version>/<timecode>/source/<arch>/
+            os.makedirs(path, exist_ok=True)
 
             buildrequest = Buildrequest({
                 "Distribution": self.dist.get(),
@@ -404,7 +407,8 @@ class Upload(Changes):
             })
 
             # Generate config files
-            config_path = path.new_sub([sbuild.CONFIG_DIR], create=True)
+            config_path = os.path.join(path, sbuild.CONFIG_DIR)
+            os.makedirs(config_path, exist_ok=True)
             config_files = {
                 sbuild.CONFIG_APT_SOURCES_LIST: repository.mbd_get_apt_build_sources_list(distribution, suite_option).get(),
                 sbuild.CONFIG_APT_PREFERENCES: repository.mbd_get_apt_build_preferences(distribution, suite_option, self.options.get("internal-apt-priority")),
@@ -414,31 +418,32 @@ class Upload(Changes):
                 sbuild.CONFIG_SBUILDRC_SNIPPET: distribution.sbuildrc_snippet,
             }
             for file_name, content in config_files.items():
-                with util.fopen(config_path.join(file_name), "w") as f:
+                with util.fopen(os.path.join(config_path, file_name), "w") as f:
                     f.write(content)
 
             # Generate tar
             tar = files.Tar()
-            tar.add([config_path.join(file_name) for file_name in config_files], arcdir=sbuild.CONFIG_DIR)
+            tar.add([os.path.join(config_path, file_name) for file_name in config_files], arcdir=sbuild.CONFIG_DIR)
             tar.add([self.dsc_file_path()])
             tar.add(files_from_dsc)
 
-            buildrequest.cset("Upload-To", util.daemon().model.mbd_get_ftp_endpoint().geturl())
+            buildrequest.cset("Upload-To", daemon.get_model().mbd_get_ftp_endpoint().geturl())
             buildrequest.cset("Packager", util.http_endpoint().geturl())
             if ao.build_architecture_all:
                 buildrequest.cset("Arch-All", "Yes")
             buildrequest.cset("Build-Dep-Resolver", distribution.get_build_dep_resolver_display())
 
-            for block in [sbuild.CONFIG_BLOCKS, sbuild.SETUP_BLOCKS]:
-                buildrequest.cset(block.name, distribution.mbd_get_extra_option(block.name, block.default))
+            buildrequest.cset(sbuild.CONFIG_BLOCKS.name, distribution.sbuild_config_blocks)
+            buildrequest.cset(sbuild.SETUP_BLOCKS.name, distribution.sbuild_setup_blocks)
 
-            buildrequest.cset("Deb-Build-Options", self.options.get("deb-build-options", ao.architecture.name, distribution.mbd_get_extra_option("Deb-Build-Options", "")))
-            buildrequest.cset("Deb-Build-Profiles", self.options.get("deb-build-profiles", ao.architecture.name, distribution.mbd_get_extra_option("Deb-Build-Profiles", "")))
-            buildrequest.cset("Add-Depends", self.options.get("add-depends", ao.architecture.name, distribution.mbd_get_extra_option("Add-Depends", "")))
+            buildrequest.cset("Deb-Build-Options", self.options.get("deb-build-options", ao.architecture.name, distribution.deb_build_options))
+            buildrequest.cset("Deb-Build-Profiles", self.options.get("deb-build-profiles", ao.architecture.name, distribution.deb_build_profiles))
+            buildrequest.cset("Add-Depends", self.options.get("add-depends", ao.architecture.name, distribution.add_depends))
 
             buildrequest.cset("Lintian-Mode", self.check_mode("lintian", ao.architecture.name, distribution.lintian_mode).mode.name)
             buildrequest.cset("Lintian-Extra-Options", distribution.lintian_extra_options)
-            buildrequest.cset("Autopkgtest-Mode", self.check_mode("autopkgtest", ao.architecture.name, int(distribution.mbd_get_extra_option("Autopkgtest-Mode", "0"))).mode.name)
+            buildrequest.cset("Lintian-Warnfail-Options", distribution.lintian_warnfail_options)
+            buildrequest.cset("Autopkgtest-Mode", self.check_mode("autopkgtest", ao.architecture.name, distribution.autopkgtest_mode).mode.name)
             buildrequest.cset("Piuparts-Mode", self.check_mode("piuparts", ao.architecture.name, distribution.piuparts_mode).mode.name)
 
             buildrequest.cset("Buildlog", os.path.join(buildrequest.bkey, self.dfn.buildlog(ao.architecture.name)))
@@ -449,7 +454,7 @@ class Upload(Changes):
                     buildrequest.cset("Builder", builder["url"])
                     buildrequest.cset("Buildlog-Building", config.URIS["builds"]["dir"].url_join(buildrequest.cget("Buildlog"), endpoint=net.ClientEndpoint(builder["url"])))
                     buildrequest.cset("Buildlog-Installed", config.URIS["events"]["dir"].url_join(buildrequest.cget("Buildlog")))
-                    buildrequest.save_to(self.events_path.new_sub([ao.architecture.name], create=True), tar)
+                    buildrequest.save_to(self.get_events_path(ao.architecture.name), tar)
                     buildrequest.upload(net.ClientEndpoint(builder["incoming_url"]))
                     buildrequest_dict[ao.architecture.name] = buildrequest
                     break
@@ -518,28 +523,28 @@ class Buildresult(Changes):
             return \
                 check(arch, lintian, "lintian", distribution.lintian_mode) and \
                 check(arch, piuparts, "piuparts", distribution.piuparts_mode) and \
-                check(arch, autopkgtest, "autopkgtest", int(distribution.mbd_get_extra_option("Autopkgtest-Mode", "0")))
+                check(arch, autopkgtest, "autopkgtest", distribution.autopkgtest_mode)
 
         return check_all() if status == "successful" else status == "skipped"
 
 
 def incoming2queue(file_path):
-    LOG.debug(f"Trying to queue incoming: {file_path}")
+    LOG.debug("Trying to queue incoming: %s", file_path)
     try:
         if Buildresult.match(file_path):
             changes = Buildresult(file_path)
-            changes.move_to_events()
-            util.daemon().packager.queue.put(changes)
+            changes.move_to(changes.get_events_path())
+            daemon.get().packager.queue.put(changes)
         elif Buildrequest.match(file_path):
-            changes = Buildrequest(file_path).create_events()
-            changes.move_to_builds()
-            util.daemon().builder.queue.put(changes)
+            changes = Buildrequest(file_path)
+            changes.move_to(config.ROUTES["builds"].path.join(changes.bkey))
+            daemon.get().builder.queue.put(changes)
         else:  # User Upload
             changes = Upload(file_path)
-            changes.move_to_events()
-            util.daemon().packager.queue.put(changes)
+            changes.move_to(changes.get_events_path())
+            daemon.get().packager.queue.put(changes)
     except Exception as e:
         # If it at least qualifies as Changes, still try error/event handling for user experience
         changes = Changes(file_path)
-        changes.move_to_events()
-        util.daemon().events.log(events.Type.REJECTED, changes, e)
+        changes.move_to(changes.get_events_path())
+        daemon.get().events.log(events.Type.REJECTED, changes, e)
