@@ -96,6 +96,11 @@ DISTRO_INFO = {
 
 APT_KEYS = {
     "Debian": {
+        "trixie": {
+            "archive": "225629DF75B188BD",    # Debian Archive Automatic Signing Key (13/trixie) <ftpmaster@debian.org>
+            "release": "762F67A0B2C39DE4",    # Debian Stable Release Key (13/trixie) <debian-release@lists.debian.org>
+            "security": "9904613D4CCE68C6",   # Debian Security Archive Automatic Signing Key (13/trixie) <ftpmaster@debian.org>
+        },
         "bookworm": {
             "archive": "B7C5D7D6350947F8",    # Debian Archive Automatic Signing Key (12/bookworm) <ftpmaster@debian.org>
             "release": "F8D2585B8783D481",    # Debian Stable Release Key (12/bookworm) <debian-release@lists.debian.org>
@@ -229,12 +234,30 @@ SETUP = {
                 "sid": {
                     "apt_keys": [APT_KEYS["Debian"]["bullseye"]["archive"], APT_KEYS["Debian"]["bookworm"]["archive"]],
                 },
+                "forky": {
+                    "apt_keys": [APT_KEYS["Debian"]["bookworm"]["archive"], APT_KEYS["Debian"]["trixie"]["archive"]],
+                    "extra_sources": [
+                        {
+                            "codename": "forky-security",
+                            "apt_keys": [APT_KEYS["Debian"]["bookworm"]["security"], APT_KEYS["Debian"]["trixie"]["security"]],
+                            "priority": 500,
+                            "options": {
+                                "remove_from_component": "updates/",
+                            },
+                        },
+                        {
+                            "origin": "Debian Backports",
+                            "codename": "forky-backports",
+                            "apt_keys": [APT_KEYS["Debian"]["bookworm"]["archive"], APT_KEYS["Debian"]["trixie"]["archive"]],
+                        },
+                    ],
+                },
                 "trixie": {
-                    "apt_keys": [APT_KEYS["Debian"]["bullseye"]["archive"], APT_KEYS["Debian"]["bookworm"]["archive"]],
+                    "apt_keys": [APT_KEYS["Debian"]["bookworm"]["archive"], APT_KEYS["Debian"]["trixie"]["archive"], APT_KEYS["Debian"]["trixie"]["release"]],
                     "extra_sources": [
                         {
                             "codename": "trixie-security",
-                            "apt_keys": [APT_KEYS["Debian"]["bullseye"]["security"], APT_KEYS["Debian"]["bookworm"]["security"]],
+                            "apt_keys": [APT_KEYS["Debian"]["bookworm"]["security"], APT_KEYS["Debian"]["trixie"]["security"]],
                             "priority": 500,
                             "options": {
                                 "remove_from_component": "updates/",
@@ -243,7 +266,7 @@ SETUP = {
                         {
                             "origin": "Debian Backports",
                             "codename": "trixie-backports",
-                            "apt_keys": [APT_KEYS["Debian"]["bullseye"]["archive"], APT_KEYS["Debian"]["bookworm"]["archive"]],
+                            "apt_keys": [APT_KEYS["Debian"]["bookworm"]["archive"], APT_KEYS["Debian"]["trixie"]["archive"]],
                         },
                     ],
                 },
