@@ -10,7 +10,7 @@ import gpg
 import gpg.constants.sig.mode
 import gpg.errors
 
-from mini_buildd import call, config, util
+from mini_buildd import call, config, daemon, util
 
 LOG = logging.getLogger(__name__)
 
@@ -58,7 +58,7 @@ class GpgmeVerifyFailed(util.HTTPUnauthorized):
     def sig_info(cls, signature):
         """Human-readable info combining actual keyid (if subkey, gpgme only provides fpr of subkey) and gpgme reason"""
         try:
-            _key, info = util.daemon().public_key_cache.keyinfo(signature.fpr)
+            _key, info = daemon.get().public_key_cache.keyinfo(signature.fpr)
             return f"{gpg.errors.GpgError(signature.status).code_str}: {info['key']} ({','.join(info['subkeys'])}): {info['user']}"
         except BaseException as e:
             LOG.warning("Can't get keyinfo for gpgme signature: %s", e)
@@ -358,7 +358,7 @@ class PublicKeyCache(TmpGnuPG):
                 self.add_pub_key(key)
             elif key_id:
                 # Import from keyserver
-                keyserver = util.daemon().model.gnupg_keyserver
+                keyserver = daemon.get_model().gnupg_keyserver
                 if not keyserver:
                     raise util.HTTPBadRequest(f"Key '{key_id}' not in installed archive keys, and no keyserver configured")
                 call.call_with_retry(self.gpg_cmd + ["--armor", "--keyserver", keyserver, "--recv-keys", key_id], retry_max_tries=5, retry_sleep=5, timeout=self.KEYSERVER_RECV_TIMEOUT)
